########################################################################
#
# File Name:            ResourceExpressions
#
#
"""
Implementation and utilities for resource procesisng Versa
WWW: http://4suite.org/4XSLT        e-mail: support@4suite.org

Copyright 2002 Fourthought Inc, USA.
See  http://4suite.org/COPYRIGHT  for license and copyright information
"""

from Ft.Xml.XPath import Util
import DataTypes
from Ft.Rdf import RDF_MS_BASE, RDF_SCHEMA_BASE, OBJECT_TYPE_RESOURCE, OBJECT_TYPE_UNKNOWN
from Ft.Rdf.Drivers import PROPERTIES

class ResourceExpression:
    """
    Base class for all resource expressions
    """
    pass

class PureQNameExpression(ResourceExpression):
    """
    Expand a QName into a string.  Uses the NsMapping on the context
    """ 
    def __init__(self,expr):
        self.expr = expr

    def evaluate(self,con):
        res = Util.ExpandQName(self.expr, namespaces=con.nsMapping)
        if res[0] is None:
            return res[1]
        return DataTypes.Resource(res[0] + res[1])


class LiteralExpression(ResourceExpression):
    """
    Use the resource as is
    """ 
    def __init__(self, value):
        self.value = value

    def evaluate(self, con):
        return self.value


class CurrentExpression(ResourceExpression):
    """
    Return the current resource if there is one
    """
    def evaluate(self,con):
        return con.current

from Ft.Lib import FtWarnings

def GetRelations(start, property, con, inverse, useSubProps=1,
                 forceTransitive=0):
    """
    Returns a dictionary of dictionaries.  The outer dict maps properties
    to relations.  The relation dicts match subjects to objects for each
    property.
    """
    property = DataTypes.ToResource(property)
    start = start and DataTypes.ToSet(start) or None
    suspects = con.driver.complete(None, str(property), None, None, con.scope, {})
    if useSubProps:
        subprops = con.getCachedSuperProps().get(property, [])
        for sp in subprops:
            suspects.extend(con.driver.complete(None, str(sp), None, None, con.scope, {}))
    sdict = {}
    if not inverse:
        for s in suspects:
            subj = DataTypes.ToResource(s[0])
            obj = s[5] == OBJECT_TYPE_RESOURCE and DataTypes.ToResource(s[2]) or s[2]
            if not sdict.has_key(subj):
                sdict[subj] = []
            sdict[subj].append(obj)
    else:
        for s in suspects:
            subj = DataTypes.ToResource(s[0])
            obj = s[5] == OBJECT_TYPE_RESOURCE and DataTypes.ToResource(s[2]) or s[2]
            if not sdict.has_key(obj):
                sdict[obj] = []
            sdict[obj].append(subj)

    #print "Raw result dict:", sdict
    #Compute transitive closure
    if forceTransitive or con.isTransitive(property):
        #if forceTransitive:
        #    print property, "treated transitively"
        #else:
        #    print property, "is transitive"
        _HandleTransitive(sdict, inverse, con)

    #print "After transitive closure:", sdict
    #Note: in the case of transitives, we have to wait until the end to
    #Cull to start nodes
    if start:
        for k in sdict.keys():
            if not start._d.has_key(k):
                del sdict[k]
    #print "After culling to start nodes:", sdict
    return sdict


def _HandleTransitive(sdict, inverse, con, checked=None, currList=None):
    if currList is None: currList = sdict.keys()
    if checked is None: checked = {}
    for subj in currList:
        #print "SUBJ:", subj, subj.__class__
        if checked.has_key(subj):
            continue
        for obj in sdict[subj]:
            #print "OBJ:", obj, obj.__class__, sdict.has_key(obj), sdict.has_key(DataTypes.ToResource(obj))
            if sdict.has_key(obj):
                if not checked.has_key(obj):
                    checked = _HandleTransitive(sdict, inverse, con, checked, [obj])
                sdict[subj].extend(sdict[obj])
        checked[subj] = None
    return checked

