########################################################################
#
# File Name:            Util.py
#
#
"""
Versa implementation utilities
WWW: http://4suite.org/4RDF        e-mail: support@4suite.org

Copyright 2002 Fourthought Inc, USA.
See  http://4suite.org/COPYRIGHT  for license and copyright information
"""


def VersaQueryGivenContext(query, context):
    from Ft.Rdf.Parsers import Versa
    exp = Versa.Compile(query)
    results = exp.evaluate(context)
    return results


def ProcessQueryFile(src, inContext, logFile=None,
                     queryFunction=VersaQueryGivenContext, scope=None):
    from Ft.Rdf.Parsers import Versa
    con = inContext.clone()
    query = ""
    for line in src.readlines():
        line = line.strip()
        if not line:continue
        if line[:5] == 'QUERY':
            query = line[6:]
        elif line[:2] == 'NS':
            name, uri = line[3:].split("=")
            con.nsMapping[name] = uri
            logFile.write("Adding namespace mapping: %s --> %s\n"%(name, uri))
        elif line[:3] == 'VAR':
            fields = line[4:].split("=")
            result = queryFunction(fields[1], con)
            name = fields[0]
            if logFile: logFile.write("Assigning variable %s using: %s\n"%(name, fields[1]))
            #FIXME: Need to handle QNames
            con.varBindings[(None, name)] = result

    if logFile: logFile.write("Executing query: %s\n"%query)
    result = queryFunction(query, con, scope)
    return result


def ResultsToXml(result, stream, indent=''):
    # By giving a namespace, the elements are created in the null-namespace
    if DataTypes.IsList(result):
        stream.write(indent)
        stream.write("<List>\n")
        for r in result:
            ResultsToXml(r, stream, indent+'  ')
        stream.write(indent)
        stream.write("</List>\n")
    elif DataTypes.IsSet(result):
        stream.write(indent)
        stream.write("<Set>\n")
        for r in DataTypes.ToList(result):
            ResultsToXml(r, stream, indent+'  ')
        stream.write(indent)
        stream.write("</Set>\n")
    elif DataTypes.IsString(result):
        stream.write(indent)
        stream.write("<String>%s</String>\n"%result)
    elif DataTypes.IsResource(result):
        stream.write(indent)
        stream.write("<Resource>%s</Resource>\n"%str(result))
    elif DataTypes.IsNumber(result):
        stream.write(indent)
        stream.write("<Number>%s</Number>\n"%result)
    elif DataTypes.IsBoolean(result):
        stream.write(indent)
        stream.write("<Boolean>%s</Boolean>\n"%result)
    else:
        raise Exception('Unknown Versa result type')
    return


import DataTypes

