########################################################################
#
# File Name:            Add.Alias.py
#
# Documentation:        http://docs.4suite.org/4SS/Lib/CommandLine.py.html
#
"""
WWW: http://4suite.org        e-mail: support@4suite.org

Copyright (c) 2001 Fourthought Inc, USA.   All Rights Reserved.
See  http://4suite.org/COPYRIGHT  for license and copyright information
"""
__doc__ = """This command will create a new alias on a resource in the system.  You will be prompted for a user name and password.  You must login as a user with write permissions. to the parent container of the new alias."""

import sys
from Ft.Server.Client import FtServerClientException
from Ft.Server.Client.Commands import CommandUtil

def Run(options,args):

    try:
        commit = 0
        repo = CommandUtil.GetRepository(options,'4ss.add.alias')
        try:
            res = repo.fetchResource(args['sourcePath'])
            res.addAlias(args['alias'])
            commit = 1
        finally:
            try:
                if commit:
                    repo.txCommit()
                else:
                    repo.txRollback()
            except:
                pass

    except FtServerClientException, e:
        sys.stderr.write(str(e))
        return


def Register():

    from Ft.Lib.CommandLine import Options, Command, Arguments

    cmd = Command.Command('alias',
                          """Add an alias to a resource in the system.""",
                          '/4Suite/Documents/documents.rdf /4Suite/document.rdf',
                          __doc__,
                          function = Run,
                          arguments = [Arguments.RequiredArgument('sourcePath',
                                                                  "The path to object being aliased.",
                                                                  str),
                                       Arguments.RequiredArgument('alias',
                                                                  "The path to the new alias object.",
                                                                  str),
                                       ],

                          fileName = __file__,
                          )
    return cmd

