########################################################################
#
# File Name:            Add.Member.py
#
# Documentation:        http://docs.4suite.org/4SS/Lib/CommandLine.py.html
#
"""
WWW: http://4suite.org        e-mail: support@4suite.org

Copyright (c) 2001 Fourthought Inc, USA.   All Rights Reserved.
See  http://4suite.org/COPYRIGHT  for license and copyright information
"""

__doc__ = """This command will add a user to a group.  You will be prompted for a user name and password.  You must login as a user with write access to the group and read access to the user.  Subgroups are allowed so the user name could be the name of another group."""

from Ft.Server.Client import FtServerClientException
from Ft.Server.Client.Commands import CommandUtil
from Ft.Server.Common import ResourceTypes

def Run(options,args):

    try:
        commit = 0
        membername = args['memberName']
        groupname = args['groupName']
        repo = CommandUtil.GetRepository(options,'4ss.add.member')
        try:
            user = repo.fetchUserOrGroupByName(membername)
            group = repo.fetchUserOrGroupByName(groupname)
            if user is not None:
                if group is not None:
                    if group.isResourceType(ResourceTypes.ResourceType.GROUP):
                        group.addMember(user)
                        commit = 1
                    else:
                        print "%s is a user, not a group." % groupname
                else:
                    print "Group %s does not exist." % groupname
            else:
                print "User %s does not exist." % membername
        finally:
            try:
                if commit:
                    repo.txCommit()
                else:
                    repo.txRollback()
            except:
                pass

    except FtServerClientException, e:
        raise


def Register():

    from Ft.Lib.CommandLine import Options, Command, Arguments

    cmd = Command.Command('member',
                          """Add a user to a group.""",
                          'developers molson',
                          __doc__,
                          function = Run,
                          arguments = [Arguments.RequiredArgument('groupName',
                                                                  "The name of the group.",
                                                                  str),
                                       Arguments.RequiredArgument('memberName',
                                                                  "The name of the user.",
                                                                  str),
                                       ],

                          fileName = __file__,
                          )
    return cmd


