########################################################################
#
# File Name:            Add.Statement.py
#
"""
WWW: http://4suite.org        e-mail: support@4suite.org

Copyright (c) 2001 Fourthought Inc, USA.   All Rights Reserved.
See  http://4suite.org/COPYRIGHT  for license and copyright information
"""

__doc__ = """Add a statement to a  model in the RDF server."""

from Ft.Server.Client import FtServerClientException
from Ft.Server.Client.Commands import CommandUtil

from Ft.Rdf import Statement, OBJECT_TYPE_LITERAL, OBJECT_TYPE_UNKNOWN

def Run(options,args):

    try:
        commit = 0
        repo = CommandUtil.GetRepository(options,'4ss.add.statement')
        try:
            model = repo.getModel()
            model.add([Statement.Statement(args['subject'],
                                           args['predicate'],
                                           args['object'],
                                           args.get('uri',''),
                                           options.get('scope',''),
                                           objectType = not options.get('obj-is-resource') and OBJECT_TYPE_LITERAL or OBJECT_TYPE_RESOURCE,
                                           )])
            commit = 1
        finally:
            try:
                if commit:
                    repo.txCommit()
                else:
                    repo.txRollback()
            except:
                pass

    except FtServerClientException, e:
        raise


def Register():

    from Ft.Lib.CommandLine import Options, Command, Arguments

    cmd = Command.Command('statement',
                          """Add a statement to the the RDF model.""",
                          'index.html http://fourthought.com#Title Fourthought',
                          __doc__,
                          function = Run,
                          arguments = [Arguments.RequiredArgument('subject',
                                                                  "The subject of the new statement.",
                                                                  str),
                                       Arguments.RequiredArgument('predicate',
                                                                  "The predicate of the new statement.",
                                                                  str),
                                       Arguments.RequiredArgument('object',
                                                                  "The object of the new statement.",
                                                                  str),
                                       Arguments.OptionalArgument('uri',
                                                                  "The uri of the new statement.",
                                                                  str),
                                       ],

                          options = Options.Options([Options.Option(None, 'scope=', "The scope of the statement"),
                                                     Options.Option(None, 'obj-is-resource=', "The object is a resource, not a literal"),
                                                     ]),
                          fileName = __file__,
                          )
    return cmd



