########################################################################
#
# File Name:            Create.Group.py
#
# Documentation:        http://docs.4suite.org/4SS/Lib/CommandLine.py.html
#
"""
WWW: http://4suite.org        e-mail: support@4suite.org

Copyright (c) 2001 Fourthought Inc, USA.   All Rights Reserved.
See  http://4suite.org/COPYRIGHT  for license and copyright information
"""

__doc__ = """This command will create a new group in 4SS."""

from Ft.Server.Client import FtServerClientException
from Ft.Server.Client.Commands import CommandUtil
from Ft.Server.Common import ResourceTypes

def Run(options,args):

    try:
        commit = 0
        repo = CommandUtil.GetRepository(options,'4ss.create.group')
        if options.has_key('container'):
            containerPath = options['container']
        else:
            containerPath = '/ftss/groups'
        try:
            container = repo.fetchResource(containerPath)
            if container.isResourceType(ResourceTypes.ResourceType.CONTAINER):
                container.createGroup(args['groupName'])
                commit = 1
            else:
                print "The resource '%s' is not a container." % container.getPath()
        finally:
            if commit:
                repo.txCommit()
            else:
                repo.txRollback()
    except FtServerClientException, e:
        raise


def Register():

    from Ft.Lib.CommandLine import Options, Command, Arguments

    # just for help message
    defaultContainer = '/ftss/groups'

    cmd = Command.Command('group',
                          'Create a new group',
                          'developers',
                          __doc__,
                          function = Run,
                          arguments = [Arguments.RequiredArgument('groupName',
                                                                  'The name of the group to create.',
                                                                  str),
                                       ],
                          options = Options.Options([Options.Option('c',
                                                                    'container=PATH',
                                                                    'The container in which to create the group (default: \'%s\')' % defaultContainer),
                                                    ]),
                          fileName = __file__,
                          )
    return cmd

