########################################################################
#
# File Name:            Create.File.py
#
# Documentation:        http://docs.4suite.org/4SS/Lib/CreateResource.py.html
#
"""
WWW: http://4suite.org        e-mail: support@4suite.org

Copyright (c) 2001 Fourthought Inc, USA.   All Rights Reserved.
See  http://4suite.org/COPYRIGHT  for license and copyright information
"""

__doc__ = "The create file commmand is used to store non-XML based data in 4SS.  If you are storing XML data please see the 'create document' command."

import sys
from Ft.Server.Client import FtServerClientException
from Ft.Lib import Uri
from Ft.Server.Client.Commands import CommandUtil

def Run(options,args):

    if args['sourceUri'] == '-':
        sourceStream = sys.stdin
    else:
        sourceStream = Uri.BASIC_RESOLVER.resolve(args['sourceUri'])

    text = sourceStream.read()

    name = args['name']
    aliases = args.get('alias',[])

    try:
        commit = 0
        repo = CommandUtil.GetRepository(options,'4ss.create.rawfile')
        try:
            doc = repo.createRawFile(name,options.get('imt','text/plain'),text)
            for alias in aliases:
                doc.addAlias(alias)

            commit = 1

        finally:
            if commit:
                repo.txCommit()
            else:
                repo.txRollback()

    except FtServerClientException, e:
        sys.stderr.write(str(e))

    if args['sourceUri'] != '-':
        sourceStream.close()



def Register():

    from Ft.Lib.CommandLine import Options, Command, Arguments

    cmd = Command.Command('rawfile',
                          'Create a new raw file',
                          '--imt=image/jpeg ./images/4Suite.jpg /images/4suite.jpg',
                          __doc__,
                          function = Run,
                          arguments = [Arguments.RequiredArgument('sourceUri',
                                                                  "The URI to read the source from (The special value '-' means 'read from stdin')",
                                                                  str),
                                       Arguments.RequiredArgument('name',
                                                                  "The name of the new document",
                                                                  str),
                                       Arguments.ZeroOrMoreArgument('alias',
                                                                    "Additional aliases to assign to the new document.",
                                                                    str),
                                       ],
                          options = Options.Options([Options.Option(None,
                                                                    'imt=',
                                                                    'The Internet media type to assign the new file. text/plain'\
                                                                    'is used as the default.'),
                                                     ]),

                          fileName = __file__
                          )
    return cmd
