########################################################################
#
# File Name:            Create.User.py
#
# Documentation:        http://docs.4suite.org/4SS/Lib/CommandLine.py.html
#
"""
WWW: http://4suite.org        e-mail: support@4suite.org

Copyright (c) 2001 Fourthought Inc, USA.   All Rights Reserved.
See  http://4suite.org/COPYRIGHT  for license and copyright information
"""

__doc__ = """This command will create a new user in 4SS.  You will be promtped for a user name and password.  You must login as a user with CREATE_USER or SUPER_USER permissions."""

import getpass, sys, sha
from Ft.Server.Client import FtServerClientException
from Ft.Server.Client.Commands import CommandUtil
from Ft.Server.Common import ResourceTypes

def Run(options,args):

    username = args['userName']

    while 1:
        p1 = getpass.getpass("Enter new password for %s: " % username)
        p2 = getpass.getpass("Re-enter password: ")
        if p1 == p2:
            password = p1
            break
        else:
            sys.stderr.write("Passwords do not match, please re-enter\n")
            
    try:
        commit = 0
        repo = CommandUtil.GetRepository(options,'4ss.create.user')
        if options.has_key('container'):
            containerPath = options['container']
        else:
            containerPath = '/ftss/users'
        try:
            container = repo.fetchResource(containerPath)
            if container.isResourceType(ResourceTypes.ResourceType.CONTAINER):
                container.createUser(username,sha.new(password).hexdigest())
                commit = 1
            else:
                print "The resource '%s' is not a container." % container.getPath()
        finally:
            try:
                if commit:
                    repo.txCommit()
                else:
                    repo.txRollback()
            except:
                pass
    except FtServerClientException, e:
        raise


def Register():

    from Ft.Lib.CommandLine import Options, Command, Arguments

    # just for help message
    defaultContainer = '/ftss/users'

    cmd = Command.Command('user',
                          'Create a new user',
                          'molson',
                          __doc__,
                          function = Run,
                          arguments = [Arguments.RequiredArgument('userName',
                                                                  'The name of the user to create.',
                                                                  str),
                                       ],
                          options = Options.Options([Options.Option('c',
                                                                    'container=PATH',
                                                                    'The container in which to create the user (default: \'%s\')' % defaultContainer),
                                                    ]),
                          fileName = __file__,
                          heading = """
Note: After entering the new user's password, you might be prompted for a user name. This is a user name of an existing user in 4SS with permissions to create a new user.
""",
                          )
    return cmd

