########################################################################
#
# File Name:            Fetch.Container.py
#
"""
WWW: http://4suite.org        e-mail: support@4suite.org

Copyright (c) 2001 Fourthought Inc, USA.   All Rights Reserved.
See  http://4suite.org/COPYRIGHT  for license and copyright information
"""

__doc__ = """
Fetch a container from the Xml server.  The output is similar to ls -la on a unix machine.  The first column of the display is the Internet Media Type.  The second is the file size, the third the date last modified, and the fourth is the relative name of the resource.
"""

import os, sys, time

from Ft.Server.Client import FtServerClientException
#from Ft.Server.Common import AclConstants
from Ft.Server.Common import ResourceTypes
from Ft.Server.Client.Commands import CommandUtil

def Run(options, args):

    repo = None
    try:
        repo = CommandUtil.GetRepository(options, '4ss.fetch.container')
        try:
            container = repo.fetchResource(args.get('path', '/'))
            if container.isResourceType(ResourceTypes.ResourceType.CONTAINER):
                sty = repo.fetchResource('/ftss/data/cl-container.xslt')
                print container.applyXslt([sty])[0]
            else:
                print "The resource '%s' is not a container." % container.getPath()
        finally:
            try:
                if repo is not None:
                    repo.txRollback()
            except:
                pass
    except FtServerClientException, e:
        sys.stderr.write(str(e) + '\n')


def Register():

    from Ft.Lib.CommandLine import Options, Command, Arguments

    cmd = Command.Command('container',
                          'List contents of a container from the repository',
                          '/fourthought.com',
                          __doc__,
                          function = Run,
                          arguments = [Arguments.OptionalArgument('path',
                                                                  'container path to view (default is /)',
                                                                  str),
                                       ],

                          fileName = __file__,
                          )
    return cmd

