########################################################################
#
# File Name:            Fetch.DocDef.py
#
# Documentation:        http://docs.4suite.org/4SS/Lib/CommandLine.py.html
#
"""
WWW: http://4suite.org        e-mail: support@4suite.org

Copyright (c) 2001 Fourthought Inc, USA.   All Rights Reserved.
See  http://4suite.org/COPYRIGHT  for license and copyright information
"""
__doc__ = """Fetch a document definition from 4SS.  To fetch the definition in XML format see serialize docdef.  You must have read access to the doc def to fetch it."""

import os, sys, time

from Ft.Server.Client import FtServerClientException
from Ft.Server.Client.Commands import CommandUtil
from Ft.Server.Common import ResourceTypes, ValidationInfo, AclConstants

def Run(options,args):
    try:
        repo = CommandUtil.GetRepository(options,'4ss.fetch.docdef')
        try:
            ddRoot = repo.getSystemDocDefContainer()
            dd = ddRoot.fetchResource(args['name'])

            print "Document Definition %s" % dd.getPath()
            print "-"*20
            print
            if dd.isResourceType(ResourceTypes.ResourceType.XSLT_DOCUMENT_DEFINITION):
                print "Transform for RDF generation:"
                print dd.getContent()
            else:
                print "Base Definitions: [%s]" % ','.join(map(lambda x:x.getUri(),dd.getBaseDefinitions()))
                print "Namespace mappings:"
                for name,value in dd.getNsMap().items():
                    print "%s --> %s" % (name,value)
                print
                print "Rdf Mappings:"
                for mapping in dd.getRdfMaps():
                    print "%s  %s  %s" % (mapping[0],mapping[1],mapping[2])
                print
                print "Read ACL: %s" % dd.getAcl(AclConstants.READ_ACCESS)
                print "Write ACL: %s" % dd.getAcl(AclConstants.WRITE_ACCESS)

                print
                cp = dd.getCreationParams()
                print "Full Text Indexing: %s" % cp.fullTextIndex
                if cp.extModules:
                    print "Xslt Extension Modules:"
                    for e in cp.extModules:
                        print "  %s" % e
                print "Enforce Schema: %s" % cp.enforceSchema

                vi = cp.validationInfo
                if vi.validationType != ValidationInfo.ValidationType.NONE:
                    raise "Finish"

        finally:
            repo.txRollback()
    except FtServerClientException, e:
        sys.stderr.write(str(e))







        return


def Register():

    from Ft.Lib.CommandLine import Options, Command, Arguments

    cmd = Command.Command('docdef',
                          'Display a document definition from the repository',
                          'BASE_XML',
                          __doc__,
                          function = Run,
                          arguments = [Arguments.RequiredArgument('name',
                                                                  'Name of a document definition to view',
                                                                  str),
                                       ],

                          fileName = __file__,
                          )
    return cmd

