########################################################################
#
# File Name:            Fetch.Group.py
#
# Documentation:        http://docs.4suite.org/4SS/Lib/CommandLine.py.html
#
"""
WWW: http://4suite.org        e-mail: support@4suite.org

Copyright (c) 2001 Fourthought Inc, USA.   All Rights Reserved.
See  http://4suite.org/COPYRIGHT  for license and copyright information
"""
__doc__ = """Fetch a group from the user server.  This will display a list of members in the group.  You need read access to the group to be able to view its members."""

import sys
from Ft.Server.Client import FtServerClientException
from Ft.Server.Client.Commands import CommandUtil
from Ft.Server.Common import AclConstants, ResourceTypes

def Run(options, args):

    try:
        name = args['groupName']
        repo = CommandUtil.GetRepository(options,'4ss.fetch.group')
        try:
            group = repo.fetchUserOrGroupByName(name)
            if group is not None:
                if group.isResourceType(ResourceTypes.ResourceType.GROUP):
                    print "Group %s:" % group.getName()
                    print "-"*10
                    print
                    print "Path: %s" % group.getPath()
                    print "Read ACL: %s" % group.getAcl(AclConstants.READ_ACCESS)
                    print "Write ACL: %s" % group.getAcl(AclConstants.WRITE_ACCESS)
                    print "Members:"
                    for m in group.getMembers():
                        print "  %s" % m.getName()
                else:
                    print "%s is a user, not a group." % name
            else:
                print "Group %s does not exist." % name
        finally:
            repo.txRollback()
    except FtServerClientException, e:
        sys.stderr.write(str(e))

        return



def Register():

    from Ft.Lib.CommandLine import Options, Command, Arguments

    cmd = Command.Command('group',
                          """List information about a group in the repository""",
                          'developers',
                          __doc__,
                          function = Run,
                          arguments = [Arguments.RequiredArgument('groupName',
                                                                  'The name of the group to fetch',
                                                                  str),
                                       ],

                          fileName = __file__,
                          )
    return cmd

