########################################################################
#
# File Name:            Fetch.Userdata.py
#
# Documentation:        http://docs.4suite.org/4SS/Lib/CommandLine.py.html
#
"""
WWW: http://4suite.org        e-mail: support@4suite.org

Copyright (c) 2001 Fourthought Inc, USA.   All Rights Reserved.
See  http://4suite.org/COPYRIGHT  for license and copyright information
"""
__doc__ = """Fetch user data from the user server.  If no keys are specified, then all of the keys associated with the user are fetched.  You must have read permissions on the user to view its keys"""

import sys
from Ft.Server.Client import FtServerClientException
from Ft.Server.Client.Commands import CommandUtil
from Ft.Server.Common import ResourceTypes

def Run(options, args):

    try:
        name = args['userName']
        repo = CommandUtil.GetRepository(options,'4ss.fetch.userdata')
        try:
            user = repo.fetchUserOrGroupByName(name)
            if user is not None and user.isResourceType(ResourceTypes.ResourceType.USER):

                if args['key']:
                    userData = {}
                    for key in args['key']:
                        userData[key] = user.get(key)
                else:
                    userData = user.getUserData()

                names = userData.keys()
                names.sort()

                print "User: %s" % args['userName']
                print "-"*30
                print
                for name in names:
                    print "%s: %s" % (name,userData[name])

            else:
                print "User %s does not exist." % name
        finally:
            repo.txRollback()
    except FtServerClientException, e:
        sys.stderr.write(str(e))



        return


def Register():

    from Ft.Lib.CommandLine import Options, Command, Arguments

    cmd = Command.Command('userdata',
                          """List name-value pairs associated with a user in the repository""",
                          'root',
                          __doc__,
                          function = Run,
                          arguments = [Arguments.RequiredArgument('userName',
                                                                  'the name of the user whose data to fetch',
                                                                  str),
                                       Arguments.ZeroOrMoreArgument('key',
                                                                    'the key (name) of a name-value pair; if none, all data for the user will be returned',
                                                                    str),
                                       ],

                          fileName = __file__,
                          )
    return cmd

