########################################################################
#
# File Name:            Rdf.Remove.py
#
"""
WWW: http://4suite.org        e-mail: support@4suite.org

Copyright 2001, Fourthought Inc, USA.
See  http://4suite.org/COPYRIGHT  for license and copyright information
"""

__doc__ = """Remove RDF statements"""

import sys
from Ft.Lib import Uri
from Ft.Rdf.Statement import Statement
from Ft.Server.Client import FtServerClientException
from Ft.Server.Client.Commands import CommandUtil
#from Ft.Server.Common.InstallLib import UnicodeFromXmlClobStream
from Ft.Rdf import Model


def Run(options, args):
    flags = {}
    if options.get('subject-regex'):
        flags['subjectFlags'] = Model.REGEX
    if options.get('predicate-regex'):
        flags['predicateFlags'] = Model.REGEX
    if options.get('object-regex'):
        flags['objectFlags'] = Model.REGEX
    subject = options.get('subject')
    predicate = options.get('predicate')
    object = options.get('object')
    try:
        commit = 0
        repo = CommandUtil.GetRepository(options, '4ss.rdf.remove')
        try:
            scope = args.get('scope')
            if options.get('document'):
                doc = repo.fetchResource(scope)
                stream = Uri.BASIC_RESOLVER.resolve(options['document'])
                src = stream.read()
                doc.deserializeAndRemove(src)
            else:
                model = repo.getModel()
                #FIXME: make requirment that they must be superuser to
                #Remove a statement without specified scope
                apply(model.removePattern,
                      (subject, predicate, object, None, scope),
                      flags)
            commit = 1
        finally:
            try:
                if commit:
                    repo.txCommit()
                else:
                    repo.txRollback()
            except:
                pass
    except FtServerClientException, e:
        sys.stderr.write(str(e))
        return


def Register():

    from Ft.Lib.CommandLine import Options, Command, Arguments

    cmd = Command.Command('remove',
                          'Remove RDF statements from a given scope',
                          '--document=spam.rdf /my/scope/path',
                          __doc__,
                          function = Run,
                          arguments = [Arguments.OptionalArgument('scope',
                                                                  "The scope from which the statements are to be removed",
                                                                  str)],
                          options = Options.Options([
                              Options.Option('d', 'document=', 'Remove statements deserialized from the given document'),
                              Options.Option(None, 'subject=', 'The subject of statements selected to be deleted'),
                              Options.Option(None, 'predicate=', 'The predicate of statements selected to be deleted'),
                              Options.Option(None, 'object=', 'The object of statements selected to be deleted'),
                              Options.Option(None, 'subject-regex', 'The subject string is a regular expression to match'),
                              Options.Option(None, 'predicate-regex', 'The predicate string is a regular expression to match'),
                              Options.Option(None, 'object-regex', 'The object string is a regular expression to match'),
                                      ]),
                          fileName = __file__,
                          )
    return cmd

