########################################################################
#
# File Name:            Remove.Statement.py
#
"""
WWW: http://4suite.org        e-mail: support@4suite.org

Copyright (c) 2001 Fourthought Inc, USA.   All Rights Reserved.
See  http://4suite.org/COPYRIGHT  for license and copyright information
"""

__doc__ = """Remove a statement from the RDF model.  You must have write access to the statement."""

from Ft.Server.Client import FtServerClientException
from Ft.Server.Client.Commands import CommandUtil

from Ft.Rdf import Statement

def Run(options,args):

    try:
        commit = 0
        repo = CommandUtil.GetRepository(options,'4ss.remove.statement')
        try:
            model = repo.getModel()
            model.remove([Statement.Statement(args['subject'],
                                              args['predicate'],
                                              args['object'],
                                              args.get('uri',''),
                                              options.get('scope',''))])


            commit = 1
        finally:
            try:
                if commit:
                    repo.txCommit()
                else:
                    repo.txRollback()
            except:
                pass
    except FtServerClientException, e:
        raise


def Register():

    from Ft.Lib.CommandLine import Options, Command, Arguments

    cmd = Command.Command('statement',
                          """Remove a statement to the the RDF model.""",
                          'index.html http://fourthought.com#Title Fourthought',
                          __doc__,
                          function = Run,
                          arguments = [Arguments.RequiredArgument('subject',
                                                                  "The subject of the statement to remove.",
                                                                  str),
                                       Arguments.RequiredArgument('predicate',
                                                                  "The predicate of the statement to remove.",
                                                                  str),
                                       Arguments.RequiredArgument('object',
                                                                  "The object of the statement to remove.",
                                                                  str),
                                       Arguments.OptionalArgument('uri',
                                                                  "The uri of the statement to remove.",
                                                                  str),
                                       ],

                          fileName = __file__,
                          )
    return cmd

