########################################################################
#
# File Name:            Set.Owner.py
#
"""

WWW: http://4suite.org        e-mail: support@4suite.org

Copyright (c) 2001 Fourthought Inc, USA.   All Rights Reserved.
See  http://4suite.org/COPYRIGHT  for license and copyright information
"""

__doc__ = """This command will change the owner of a resource.  You must have change owner permissions for the given resource."""

import sys
from Ft.Server.Client import FtServerClientException
from Ft.Server.Client.Commands import CommandUtil
from Ft.Server.Common import AclConstants, ResourceTypes


def Run(options, args):
    try:
        commit = 0
        repo = CommandUtil.GetRepository(options, '4ss.set.owner')
        try:
            name = args['newowner']
            res = repo.fetchResource(args['path'], traverseAliases=0)
            user = repo.fetchUserOrGroupByName(name)
            if user is not None:
                if user.isResourceType(ResourceTypes.ResourceType.USER):
                    res.setOwner(user)
                else:
                    sys.stderr.write("%s is a group, not a user.\n" % name)
            else:
                sys.stderr.write("User %s does not exist.\n" % name)
            commit = 1
        finally:
            try:
                if commit:
                    repo.txCommit()
                else:
                    repo.txRollback()
            except:
                pass
    except FtServerClientException, e:
        sys.stderr.write(str(e))


def Register():

    from Ft.Lib.CommandLine import Options, Command, Arguments

    cmd = Command.Command('owner',
                          'Change the owner',
                          '/documents',                          
                          __doc__,
                          function=Run,
                          arguments=[Arguments.RequiredArgument('path',
                                                                'container path to create',
                                                                str),
                                     Arguments.RequiredArgument('newowner',
                                                                'user name of the new owner',
                                                                str),
                                     ],
                          options=Options.Options([
                                                   ]),
                          fileName=__file__,
                          )
    return cmd

