########################################################################
#
# File Name:            Set.UserData.py
#
# Documentation:        http://docs.4suite.org/4SS/Lib/CommandLine.py.html
#
"""
WWW: http://4suite.org        e-mail: support@4suite.org

Copyright (c) 2001 Fourthought Inc, USA.   All Rights Reserved.
See  http://4suite.org/COPYRIGHT  for license and copyright information
"""


__doc__ = """This command will set a name value pair on the user.  You must have write access to the user (or SUPER_USER permissions)."""

from Ft.Server.Client import FtServerClientException
from Ft.Server.Client.Commands import CommandUtil
from Ft.Server.Common import ResourceTypes

def Run(options,args):

    try:
        name = args['userName']
        commit = 0
        repo = CommandUtil.GetRepository(options,'4ss.set.userdata')
        try:
            user = repo.fetchUserOrGroupByName(name)
            if user is not None and user.isResourceType(ResourceTypes.ResourceType.USER):
                if args.has_key('value'):
                    user[args['key']] = args['value']
                else:
                    del user[args['key']]
                commit = 1
            else:
                print "User %s does not exist." % name
        finally:
            try:
                if commit:
                    repo.txCommit()
                else:
                    repo.txRollback()
            except:
                pass
    except FtServerClientException, e:
        raise


def Register():

    from Ft.Lib.CommandLine import Options, Command, Arguments

    cmd = Command.Command('userdata',
                          'Associate a name-value pair with a user.',
                          'molson age 26',
                          __doc__,
                          function = Run,
                          arguments = [Arguments.RequiredArgument('userName',
                                                                  "the name of the user on which to set data",
                                                                  str),
                                       Arguments.RequiredArgument('key',
                                                                  "the key (name) for the name-value pair",
                                                                  str),
                                       Arguments.OptionalArgument('value',
                                                                  "the value for the name-value pair; no value means delete the key",
                                                                  str),
                                       ],
                          fileName = __file__,
                          )
    return cmd


