########################################################################
#
# File Name:            Update.File.py
#
#
"""
Update a resource int the repository
WWW: http://4suite.org        e-mail: support@4suite.org

Copyright (c) 2001 Fourthought Inc, USA.   All Rights Reserved.
See  http://4suite.org/COPYRIGHT  for license and copyright information
"""

__doc__ = """Update the content of a Raw File in the 4Suite Repository"""

import sys
from Ft.Server.Client import FtServerClientException
from Ft.Lib import Uri

from Ft.Server.Client.Commands import CommandUtil

def Run(options,args):

    if args['sourceUri'] == '-':
        sourceStream = sys.stdin
    else:
        sourceStream = Uri.BASIC_RESOLVER.resolve(args['sourceUri'])

    text = sourceStream.read()

    try:
        commit = 0
        repo = CommandUtil.GetRepository(options,'4ss.update.rawfile')
        try:
            doc = repo.fetchResource(args['path'])
            doc.setContent(text)
            if options.get('imt'):
                doc.setImt(options['imt'])
            commit = 1
        finally:
            try:
                if commit:
                    repo.txCommit()
                else:
                    repo.txRollback()
            except:
                pass

    except FtServerClientException, e:
        sys.stderr.write(str(e))

    if args['sourceUri'] != '-':
        sourceStream.close()

def Register():

    from Ft.Lib.CommandLine import Options, Command, Arguments

    cmd = Command.Command('rawfile',
                          """Update a raw file.""",
                          'localfile.jpg /images/4Suite.jpg',
                          __doc__,
                          function = Run,
                          arguments = [
                                       Arguments.RequiredArgument('sourceUri',
                                                                  "The URI to read the source from (Use "\
                                                                  "'-' to read it from standard input)",
                                                                  str),
                                       Arguments.RequiredArgument('path',
                                                                  "The path of the existing raw file in "\
                                                                  "the repository.",
                                                                  str),
                                       ],
                          options = Options.Options([Options.Option(None,
                                                                    'imt=','The internet media type to assign to the '\
                                                                    'raw file.  If not specified it is left unchanged.'),
                                                     ]),
                          fileName = __file__,
                          )
    return cmd


