########################################################################
#
# File Name:            FtssInputSource.py
#
# Documentation:        http://docs.4suite.org/FtssInputSource.html
#
"""

WWW: http://4suite.org        e-mail: support@4suite.org

Copyright (c) 1999-2001 Fourthought Inc, USA.   All Rights Reserved.
See  http://4suite.org/COPYRIGHT  for license and copyright information
"""

import urlparse, cStringIO
from Ft.Xml import InputSource, Domlette
import PathImp
from Ft.Lib import Uri, Uuid, FtWarnings

class FtssInputSource(InputSource.InputSource):
    """
    Base class for the various repository readers.
    Reads content from the repository using the specified path
    """
    def __init__(self, stream, uri, driver, *v_args, **kw_args):
        if uri:
            self.uri = uri
            self.baseParts = urlparse.urlparse(uri)
        else:
            self.uri = 'urn:uuid:' + Uuid.UuidAsString(Uuid.GenerateUuid())
            self.baseParts = None
        self._driver = driver
        return apply(InputSource.InputSource.__init__,
                     (self, stream, uri) + v_args,
                     kw_args)

    def _openStream(self, uri):
        parts = urlparse.urlparse(uri)
        if not self.baseParts or parts[0:2] != self.baseParts[0:2]:
            return InputSource.InputSource._openStream(self, uri)

        #Make it a path
        path = PathImp.CreateInitialPath('/',self._driver)
        path = path.normalize(uri,1)
        if self._driver.hasResource(path):
            return cStringIO.StringIO(self._driver.fetchResource(path))
        return InputSource.InputSource._openStream(self, uri)

    def _clone(self, stream, uri):
        return FtssInputSource(stream, uri, self._driver,
                               processIncludes=self.processIncludes,
                               stripElements=self.stripElements)

    def __getstate__(self):
        #Must not pickle the driver
        state = InputSource.InputSource.__getstate__(self)
        state['_driver'] = None
        return state
            

FtssInputSourceFactory = InputSource.InputSourceFactory(FtssInputSource)

ValidatingReader = Domlette.ValidatingReaderBase(FtssInputSourceFactory)
NonvalidatingReader = Domlette.NonvalidatingReaderBase(FtssInputSourceFactory)
