import time, urllib

import RawFileImp
from Ft.Server import FTSERVER_NAMESPACE
from Ft.Server.Common import Schema
from Ft.Server.Common.ResourceTypes import ResourceType

from Ft.Server.Server import FtServerServerException, Error
from Ft.Server.Server.Drivers import FtssDriver

class UriReferenceFileImp(RawFileImp.RawFileImp):
    """
    An Uri Reference File is a reference to a URI.  You are given read only access to it.
    MetaData about the Content is updated every time you call getContent
    """
    resourceType = ResourceType.URI_REFERENCE_FILE

    def getReference(self):
        self._verifyTx()

        res = self._driver.getSystemModel().complete(self._path.absolutePath,Schema.URI_REFERENCE_LOCATION,None)
        if not res:
            raise FtServerServerException(Error.UNKNOWN_PATH,
                                          path = self._path)
        return res[0].object

    def setContent(self,*args):
        raise FtServerServerException(Error.PERMISSION_DENIED,
                                      level='write',
                                      path = self._path)

    def getContent(self):

        content = urllib.urlopen(self.getReference()).read()

        from Ft.Server.Server.Drivers import FtssDriver
        xu = FtssDriver.UPDATE_SIZE % (FTSERVER_NAMESPACE, (len(content)))
        self._driver.xupdateMetaData(self._path, xu)
        return content




def NewUriReferenceFileXml(driver,path,acl,owner,imt,srcUri):
    keywords = {
        'ns' : FTSERVER_NAMESPACE,
        'path' : path,
        'type' : Schema.g_rdfResourceTypes[ResourceType.URI_REFERENCE_FILE],
        'dt' : FtssDriver.CurrentTime(),
        'acl' : driver.aclToXml(acl),
        'owner' : owner,
        'media-type' : imt,
        'size' : len(urllib.urlopen(srcUri).read()),
        'reference' : srcUri,
        }

    xml = """\
<ftss:MetaData xmlns:ftss='%(ns)s' path='%(path)s' type='%(type)s' creation-date='%(dt)s'>
  %(acl)s
  <ftss:LastModifiedDate>%(dt)s</ftss:LastModifiedDate>
  <ftss:Owner>%(owner)s</ftss:Owner>
  <ftss:Imt>%(media-type)s</ftss:Imt>
  <ftss:Size>%(size)d</ftss:Size>
  <ftss:Reference>%(reference)s</ftss:Reference>
</ftss:MetaData>""" % keywords

    return xml
