########################################################################
#
# File Name:            User.py
#
"""
XSLT and XPath extensions supporting the 4SS User API
WWW: http://4suite.org        e-mail: support@4suite.org

Copyright (c) 2000-2001 Fourthought Inc, USA.   All Rights Reserved.
See  http://4suite.org/COPYRIGHT  for license and copyright information
"""

import sha
from Ft.Lib import boolean
from Ft.Server import FTSERVER_NAMESPACE
from Ft.Server.Common import ResourceTypes
from Ft.Xml.XPath import Conversions
from Ft.Xml.Xslt import XsltElement, ContentInfo, AttributeInfo

from Ns import SCORE_NS
import FtssXsltBase

def SetUserData(context, key, value):
    key = Conversions.StringValue(key)
    value = Conversions.StringValue(value)
    repo = context.processor._repository
    user = repo.getCurrentUser()
    if user is not None:
        if user.has_key(key):
            print "Xslt/User: Value '" + value + "' replaces value '" + user[key] + "' for key '" + key + "' in user data."
        user[key] = value
        context.processor.extensionParams[(FTSERVER_NAMESPACE,'commit')]=1
        return boolean.true
    else:
        print "Xslt/User: set-user-data() called but there is no current repo user."
        return boolean.false
        
def GetCurrentUser(context):
    repo = context.processor._repository    
    user = repo.getCurrentUser()
    if user is None:
        return ''
    else:
        return  user.getAbsolutePath()
    
def GetUserName(context):    
    repo = context.processor._repository    
    user = repo.getCurrentUser()
    if user is None:
        return "Anonymous"  # should be 'world'?
    else:
        return user.getUsername()
    
def GetUserOrGroupByName(context,name):
    repo = context.processor._repository
    user_or_group = repo.fetchUserOrGroupByName(name)
    if user_or_group is None:
        return ''
    else:
        return user_or_group.getAbsolutePath()
    
def GetUserData(context, key):
    key = Conversions.StringValue(key)
    repo = context.processor._repository
    user = repo.getCurrentUser()
    if user is not None:
        if user.has_key(key):
           return user[key]
        else:
            print "Xslt/User: Key '" + key + "' not found in user data."
    else:
        print "Xslt/User: get-user-data() called but there is no current repo user."
    return ''

def RemoveUserData(context,key):
    key = Conversions.StringValue(key)
    repo = context.processor._repository
    user = repo.getCurrentUser()
    if user is not None:
        if user.has_key(key):
            print "Xslt/User: Key '" + key + "' with value '" + user[key] + "' removed from user data."
            del user[key]
            context.processor.extensionParams[(FTSERVER_NAMESPACE,'commit')]=1
            return boolean.true
        else:
            print "Xslt/User: Key '" + key + "not found in user data."
    else:
        print "Xslt/User: remove-user-data() called but there is no current repo user."
    return boolean.false


class ChangeUserPasswordElement(XsltElement):
    """
    Changes the password of the specified user
    """

    content = ContentInfo.Empty

    legalAttrs = {
        'path' : AttributeInfo.UriReferenceAvt(required=1,
                                               description='The path of the user'),
        'new-password' : AttributeInfo.StringAvt(default=None,
                                                 description='The new password. If '
                                                 'not specified, will remove the '
                                                 'password for this user.'),
        }
    childArgument = None

    def instantiate(self, context, processor):
        context.setProcessState(self)

        path = self._path.evaluate(context)
        password = self._new_password.evaluate(context) or ''

        hashedPW = sha.new(password).hexdigest()

        user = FtssXsltBase.FetchBaseObject(processor, path)
        user.setPassword(hashedPW)

        return (context,)



ExtFunctions = {
    (SCORE_NS, 'get-user-data'):     GetUserData,
    (SCORE_NS, 'get-current-user'):  GetCurrentUser,
    (SCORE_NS, 'get-user-name'):     GetUserName,
    (SCORE_NS, 'get-user-or-group'): GetUserOrGroupByName,
    (SCORE_NS, 'set-user-data'):     SetUserData,
    (SCORE_NS, 'remove-user-data'):  RemoveUserData,
}

ExtElements = {
    (SCORE_NS, 'change-password'): ChangeUserPasswordElement,
}
