import urllib, os
from Ft.Lib import Uri

# Test cases for resolution with a base URI
#
test_cases = [
    ('','file:docs/xml/4XLink.api','file:docs/xml/4XLink.api'),
    ('file:docs/xml/','file:docs/xml/4XLink.api','file:docs/xml/4XLink.api'),
    ('file:/usr/local/lib/xslt/bar.xslt','http://4Suite.org/foo.xslt','http://4Suite.org/foo.xslt'),
    ]

# Test cases for URI reference syntax;
# some of these URIs come from a Perl script by David A. Wheeler
# at http://www.dwheeler.com/secure-programs/url.pl
#
good_URIs = [
    'http://www.yahoo.com',
    'http://www.yahoo.com/',
    'http://1.2.3.4/',
    'http://www.yahoo.com/stuff',
    'http://www.yahoo.com/stuff/',
    'http://www.yahoo.com/hello%20world/',
    'http://www.yahoo.com?name=obi',
    'http://www.yahoo.com?name=obi+wan&status=jedi',
    'http://www.yahoo.com?onery',
    'http://www.yahoo.com#bottom',
    'http://www.yahoo.com/yelp.html#bottom',
    'https://www.yahoo.com/',
    'ftp://www.yahoo.com/',
    'ftp://www.yahoo.com/hello',
    'demo.txt',
    'demo/hello.txt',
    'demo/hello.txt?query=hello#fragment',
    '/cgi-bin/query?query=hello#fragment',
    '/demo.txt',
    '/hello/demo.txt',
    'hello/demo.txt',
    '/',
    '',
    '#here',
    'http://www.yahoo.com?name=%00%01',
    'http://www.yaho%6f.com',
    'http://www.yahoo.com/hello%00world/',
    'http://www.yahoo.com/hello+world/',
    'http://www.yahoo.com?name=obi&',
    'http://www.yahoo.com?name=obi&type=',
    'http://www.yahoo.com/yelp.html#',
    '//',
    'file:///foo/bar',
    'mailto:user@host?subject=blah',
    ]

bad_URIs = [
    'http://www yahoo.com',
    'http://www.yahoo.com/hello world/',
    'http://www.yahoo.com/yelp.html#"',
    '::',
    'beepbeep\x07\x07',
    '\n',
    ]

def Test(tester):

    tester.startGroup("urllib basejoin")

    #resolver = Uri.BaseUriResolver()
    for base, relative, expectedUri  in test_cases:
        tester.startTest("URI: base=%s uri=%s" % (base,relative))

        res = urllib.basejoin(base, relative)
        tester.compare(expectedUri, res, "Invalid Uri")

        tester.testDone()

    tester.groupDone()

    tester.startGroup("URI reference syntax")
    for testuri in good_URIs:
        tester.startTest("Good URI: %s" % repr(testuri))
        tester.compare(1, Uri.matchesUriRefSyntax(testuri), "Tests Bad!")
        tester.testDone()
    for testuri in bad_URIs:
        tester.startTest("Bad URI: %s" % repr(testuri))
        tester.compare(0, Uri.matchesUriRefSyntax(testuri), "Tests Good!")
        tester.testDone()
    tester.groupDone()

    tester.startGroup("Basic Uri Resolver")
    data = [('http://foo.com/root/','path','http://foo.com/root/path'),
            ('http://foo.com/root','path','http://foo.com/path'),
            ]
    for base,uri,exp in data:
        tester.startTest("normalize: %s %s" % (base,uri))
        res = Uri.BASIC_RESOLVER.normalize(uri,base)
        tester.compare(exp,res)
        tester.testDone()

    base = 'foo:foo.com'
    uri = 'path'
    tester.startTest("normalize: %s %s" % (base,uri))
    tester.testException(Uri.BASIC_RESOLVER.normalize,(uri,base),Uri.UriException)
    tester.testDone()

    tester.startTest('resolve')
    base = os.getcwd()
    if base[-1] != os.sep:
        base += os.sep
    stream = Uri.BASIC_RESOLVER.resolve('test.py',base)
    tester.compare("#!/usr/bin/env python\n",stream.readline())
    stream.close()
    tester.testDone()

    tester.startTest('generate')
    uuid = Uri.BASIC_RESOLVER.generate()
    tester.compare('urn:uuid:',uuid[:9])

    tester.testDone()

    tester.groupDone()


if __name__ == '__main__':
    from Ft.Lib.TestSuite import Tester
    tester = Tester.Tester()
    Test(tester)
