########################################################################
#
# File Name:            Domlette.py
#
#
"""
Abstraction module.  Will try to use cDomlette as much as possible
WWW: http://4suite.org/4XSLT        e-mail: support@4suite.org

Copyright (c) 1999-2001 Fourthought Inc, USA.   All Rights Reserved.
See  http://4suite.org/COPYRIGHT  for license and copyright information
"""

import os, sys, cStringIO, warnings
import InputSource
import codecs, encodings
from types import UnicodeType

MINIDOM = int(os.environ.get("USE_MINIDOM", "0"))


from Lib.Nss import GetAllNs

if MINIDOM:
    from FtMiniDom import XmlStrStrip, XmlStrRStrip, XmlStrLStrip
    from FtMiniDom import implementation, Node
    from FtMiniDom import nonvalParse, valParse
    from Lib.Print import Print, PrettyPrint
    from Lib.Nss import SeekNss
    
else:
    from cDomlette import nonvalParse, valParse
    from cDomlette import XmlStrStrip, XmlStrRStrip, XmlStrLStrip
    from cDomlette import implementation, Node

    #NOTE, there is no print ro in C yet so use the python versions
    from Lib.Nss import SeekNss

    #Note, there is no Print and PrettyPrint in c yet so use the python versions
    from Lib.Print import Print, PrettyPrint

class _Reader:
    """
    Now there is one reader to do everything
    """
    def __init__(self, parseMethod, inputSourceFactory=None):
        self.inputSourceFactory = inputSourceFactory or InputSource.DefaultFactory
        self.parseMethod = parseMethod

    def parse(self, inputSource):
        """
        Parse in a input source and return a DOM
        """
        if not isinstance(inputSource, InputSource.InputSource):
            raise TypeError("inputsource must be an instance of "
                            "Ft.Xml.InputSource.InputSource")
        return self.parseMethod(inputSource)


    def parseString(self, st, uri, *v_args, **kw_args):
        if not uri:
            warnings.warn("Creation of InputSource without an URI",
                          RuntimeWarning, 2)
        isrc = apply(self.inputSourceFactory.fromString, (st, uri) + v_args, kw_args)
        return self.parse(isrc)

    def parseStream(self, stream, uri, *v_args, **kw_args):
        if not uri:
            warnings.warn("Creation of InputSource without an URI",
                          RuntimeWarning, 2)
        isrc = apply(self.inputSourceFactory.fromStream, (stream,uri) + v_args, kw_args)
        return self.parse(isrc)

    def parseUri(self, uri, *v_args, **kw_args):
        isrc = apply(self.inputSourceFactory.fromUri, (uri,) + v_args, kw_args)
        return self.parse(isrc)
    
class ValidatingReaderBase(_Reader):
    """
    Base class to be used by all validating readers
    """
    def __init__(self,inputSourceFactory=None):
        _Reader.__init__(self, valParse, inputSourceFactory)

class NonvalidatingReaderBase(_Reader):
    """
    Base class to be used by all non validating readers
    """
    def __init__(self,inputSourceFactory=None):
        _Reader.__init__(self, nonvalParse, inputSourceFactory)


#Create the instances of them
ValidatingReader = ValidatingReaderBase()
NonvalidatingReader = NonvalidatingReaderBase()

class DeprecatedReader:
    """
    Defined to support the old interfaces
    """
    def __init__(self,resolveEntity = None, processIncludes = 1):
        warnings.warn("You are using deprecated readers",
                      DeprecationWarning, 2)
        self.resolveEntity = resolveEntity
        self.processIncludes = processIncludes
        self.inputSourceFactory = InputSource.DefaultFactory
        
    def fromUri(self, uri, baseUri='', ownerDoc=None, stripElements=None):
        """
        Create a default input Source
        """
        if not baseUri:
            warnings.warn("Creation of InputSource without an URI",
                          RuntimeWarning, 2)
        src = InputSource.InputSource(None, uri,
                                      processIncludes=self.processIncludes,
                                      stripElements=stripElements)
        src = src.resolve(uri, '', '')
        return self._parseMethod[0](src)

    def fromStream(self, stream, refUri='', ownerDoc=None,
                   stripElements=None):
        """
        Create an input source
        """
        if not refUri:
            warnings.warn("Creation of InputSource without an URI",
                          RuntimeWarning, 2)
        src = InputSource.InputSource(stream, refUri,
                                      processIncludes=self.processIncludes,
                                      stripElements=stripElements)
        return self._parseMethod[0](src)


    encoder = codecs.lookup("utf-8")[0] # encode,decode,reader,writer
    def fromString(self, st, refUri='', ownerDoc=None, stripElements=None):
        "Create a DOM from a string"

        if type(st) is UnicodeType:
            st = self.encoder(st)[0]
        stream =  cStringIO.StringIO(st)

        if not refUri:
            warnings.warn("Creation of InputSource without an URI",
                          RuntimeWarning, 2)
        rt = self.fromStream(stream, refUri, ownerDoc, stripElements)
        stream.close()
        return rt

class DEFAULT_NONVALIDATING_READER(DeprecatedReader):
    _parseMethod = (nonvalParse,)

class DEFAULT_VALIDATING_READER(DeprecatedReader):
    _parseMethod = (valParse,)



