########################################################################
#
# File Name:            NonvalReader.py
#
#
"""

Definition of the Non-validating reader for minidom

WWW: http://4suite.org/4XSLT        e-mail: support@4suite.org

Copyright (c) 1999-2001 Fourthought Inc, USA.   All Rights Reserved.
See  http://4suite.org/COPYRIGHT  for license and copyright information
"""

from Ft.Xml import VerifyPyexpat
VerifyPyexpat("minidom's non-validating reader", dtdSupport=1)

from xml.parsers import expat
from Ft.Xml import ReaderException

import Handler


class NonvalReader(Handler.Handler):

    ### Reader Interface ###

    def fromSrc(self, source):
        self.prepareParser()
        self.initHandler(source)

        self.parse(source)

        self.reset()
        return self.getRootNode()

    def prepareParser(self):
        parser = self.parser = expat.ParserCreate()
        parser.SetParamEntityParsing(expat.XML_PARAM_ENTITY_PARSING_ALWAYS)
        parser.StartElementHandler = self.startElement
        parser.EndElementHandler = self.endElement
        parser.CharacterDataHandler = self.characters
        parser.ProcessingInstructionHandler = self.processingInstruction
        parser.CommentHandler = self.comment
        parser.ExternalEntityRefHandler = self.externalEntityRef

        parser.StartDoctypeDeclHandler = self.startDoctypeDecl
        return

    def parse(self, source):
        self.parser.SetBase(source.uri)
        try:
            self.parser.ParseFile(source.stream)
        except expat.error:
            raise ReaderException(ReaderException.XML_PARSE_ERROR,
                                  self.parser.GetBase(),
                                  self.parser.ErrorLineNumber,
                                  self.parser.ErrorColumnNumber,
                                  expat.ErrorString(self.parser.ErrorCode))

        return

    def startDoctypeDecl(self, name, systemId, publicId, hasInternalSubset):
        # Disable comment and processing instructions while reading DTD
        self.parser.CommentHandler = None
        self.parser.ProcessingInstructionHandler = None
        self.parser.EndDoctypeDeclHandler = self.endDoctypeDecl
        return

    def endDoctypeDecl(self):
        # Turn them back on
        self.parser.CommentHandler = self.comment
        self.parser.ProcessingInstructionHandler = self.processingInstruction
        return

    def externalEntityRef(self, context, base, sysid, pubid):
        source = self.inputSource.resolve(sysid, pubid, 'EXTERNAL_ENTITY')

        orig_parser = self.parser
        orig_source = self.inputSource

        self.inputSource = source
        self.parser = self.parser.ExternalEntityParserCreate(context)
        self.parser.SetBase(source.uri)
        self.parser.ParseFile(source.stream)
        source.stream.close()

        self.inputSource = orig_source
        self.parser = orig_parser
        return 1


    def reset(self):
        self.parser = None
        return


def nonvalParse(src):
    r = NonvalReader()
    return r.fromSrc(src)

