import re

from types import UnicodeType

def TranslateCdataAttr(characters, encoding='UTF-8'):
    '''Handles normalization and some intelligence about quoting'''
    if not characters:
        return '', "'"
    if u"'" in characters:
        delimiter = '"'
        new_chars = re.sub(u'"', u'&quot;', characters)
    else:
        delimiter = "'"
        new_chars = re.sub(u"'", u'&apos;', characters)
    #Normalize
    if u"\n" in characters:
        new_chars = re.sub(u'\n', u'&#10;', new_chars)
    #FIXME: Need an XML-specific splitter to avoid platform-specific
    #whitespace selection
    new_chars = " ".join(new_chars.split())
    new_chars_str = TranslateCdata(new_chars, encoding)
    return new_chars_str, delimiter


def TranslateCdata(characters, encoding='UTF-8', prev_chars='', markupSafe=0):
    """Takes an input string (Unicode or encoded) and returns an encoded string.
    Intended primarily for XML output.
    If a character cannot be represented directly in the desired encoding,
    it is escaped as a decimal character reference like '&#1234;'.
    If the string contains characters that are not allowed in XML, those
    characters are each replaced with '&#63;'. (Emitting a character reference
    rather than a literal '?' should help indicate to the user that something
    was amiss with their XML).
    If markupSafe is 0, the following changes are made to the string:
      1. '&' becomes '&amp;'.
      2. '<' becomes '&lt;'.
      3. ']]>' becomes ']]&gt;'.
    """
    if not characters:
        return ''
    if type(characters) is not UnicodeType:
        characters = unicode(characters, encoding)
    if not markupSafe:
        if CDATA_CHAR_PATTERN.search(characters):
            new_string = CDATA_CHAR_PATTERN.subn(
                lambda m, d=CHAR_TO_ENTITY: d[m.group()],
                characters)[0]
        else:
            new_string = characters
        if prev_chars[-2:] == ']]' and characters[0] == '>':
            new_string = u'&gt;' + new_string[1:]
    else:
        new_string = characters
    #In XML output, you are not allowed to emit an illegal character,
    #not even as a character reference.
    if XML_ILLEGAL_CHAR_PATTERN.search(new_string):
        #new_string = XML_ILLEGAL_CHAR_PATTERN.subn(
        #    lambda m: '&#%i;' % ord(m.group()),
        #    new_string)[0]
        new_string = XML_ILLEGAL_CHAR_PATTERN.subn(u'&#63;', new_string)[0]
    #Test whether this string can be encoded as desired
    try:
        new_string = new_string.encode(encoding)
    except:
        new_string = UseNumericalCharEntities(new_string, encoding)
    return new_string

from HtmlDefs import *
ILLEGAL_LOW_CHARS = u'[\u0001-\u0008\u000B-\u000C\u000E-\u001F]'
#Python really doesn't like this next line, and fails very strangely.
#It will import just fine from the .py, but not the .pyc
#ILLEGAL_HIGH_CHARS = u'[\uD800-\uDFFF\uFFFE\uFFFF]'
ILLEGAL_HIGH_CHARS = u'[\uFFFE\uFFFF]'
XML_ILLEGAL_CHAR_PATTERN = re.compile(u'%s|%s'%(ILLEGAL_LOW_CHARS, ILLEGAL_HIGH_CHARS))

CDATA_CHAR_PATTERN = re.compile('[&<]|]]>')
CHAR_TO_ENTITY = {
        '&': u'&amp;',
        '<': u'&lt;',
        ']]>': u']]&gt;',
        }


def UseNumericalCharEntities(uText, encoding):
    #FIXME: This is a work-around, contributed by Mike Brown, that
    #Deals with escaping output, until we have XML/HTML aware codecs
    encoded_text = ''
    for c in uText:
        try:
            c.encode(encoding)
        except:
            #Note: use decimal char ref because some browsers are broken
            c = u'&#%i;' % ord(c)
        encoded_text += c.encode(encoding)
    return encoded_text


def ConvertChar(m):
    return u'&'+HTML_CHARACTER_ENTITIES[ord(m.group())]+u';'

def UseHtmlCharEntities(text, encoding):
    new_text, num_subst = re.subn(g_htmlUniCharEntityPattern, ConvertChar,
                                  text)
    return new_text

def TranslateHtmlCdata(characters, encoding='UTF-8', prev_chars=''):
    if type(characters) is not UnicodeType:
        characters = unicode(characters, encoding)
    #Translate numerical char entity references with HTML entity equivalents
    new_string, num_subst = re.subn(
        CDATA_CHAR_PATTERN,
        lambda m, d=CHAR_TO_ENTITY: d[m.group()],
        characters
        )
    if prev_chars[-2:] == ']]' and new_string[0] == '>':
        new_string = '&gt;' + new_string[1:]
    new_string = UseHtmlCharEntities(new_string, encoding)
    #Test whether this string can be encoded as desired
    try:
        new_string.encode(encoding)
    except:
        new_string = UseNumericalCharEntities(new_string, encoding)
    return new_string


