from XmlPrinter import XmlPrinter

class XmlPrettyPrinter(XmlPrinter):

    # The amount of indent for each level of nesting
    indent = '  '
    
    def __init__(self, stream, encoding):
        XmlPrinter.__init__(self, stream, encoding)
        self.level = 0
        self.indentAllowed = 0  # don't indent first element
        self.indentEndTag = 1
        return

    def startElement(self, tagName, namespaces, attributes):
        if self.inElement:
            self.writeAscii('>')
            self.inElement = 0
        if self.indentAllowed:
            self.writeAscii('\n' + (self.indent * self.level))
        XmlPrinter.startElement(self, tagName, namespaces, attributes)
        self.level += 1
        self.indentAllowed = 1
        return

    def endElement(self, tagName):
        self.level -= 1
        # Do not break short tag form (<tag/>)
        if self.indentAllowed and not self.inElement:
            self.writeAscii('\n' + (self.indent * self.level))
        XmlPrinter.endElement(self, tagName)
        # Allow indenting after endtags
        self.indentAllowed = 1
        return

    def text(self, data, disableEscaping=0):
        XmlPrinter.text(self, data)
        # Do not allow indenting for elements with mixed content
        self.indentAllowed = 0
        return

    def cdataSection(self, data):
        XmlPrinter.cdataSection(self, data)
        # Do not allow indenting for elements with mixed content
        self.indentAllowed = 0
        return

    def processingInstruction(self, target, data):
        if self.inElement:
            self.writeAscii('>')
            self.inElement = 0
        if self.indentAllowed:
            self.writeAscii('\n' + (self.indent * self.level))
        XmlPrinter.processingInstruction(self, target, data)
        # Allow indenting after processing instructions
        self.indentAllowed = 1
        return

    def comment(self, data):
        if self.inElement:
            self.writeAscii('>')
            self.inElement = 0
        if self.indentAllowed:
            self.writeAscii('\n' + (self.indent * self.level))
        XmlPrinter.comment(self, data)
        # Allow indenting after comments
        self.indentAllowed = 1
        return
