########################################################################
#
# File Name:            AttributeElement.py
#
# Documentation:        http://docs.4suite.org/4XSLT/AttributeElement.py.html
#
"""
Implementation of the XSLT Spec element stylesheet element.
WWW: http://4suite.org/4XSLT        e-mail: support@4suite.org

Copyright (c) 1999-2001 Fourthought, Inc., USA.   All Rights Reserved.
See  http://4suite.org/COPYRIGHT  for license and copyright information
"""

from Ft.Xml import EMPTY_NAMESPACE, SplitQName
from Ft.Xml.Xslt import XsltElement, XsltRuntimeException, Error, XSL_NAMESPACE
from Ft.Xml.Xslt import CategoryTypes, ContentInfo, AttributeInfo

class AttributeElement(XsltElement):

    category = CategoryTypes.INSTRUCTION
    content = ContentInfo.Template
    legalAttrs = {
        'name' : AttributeInfo.QNameAvt(required=1,noExpand=1),
        'namespace' : AttributeInfo.UriReferenceAvt(),
        }

    def instantiate(self, context, processor):
        context.processorNss = self.namespaces
        context.currentInstruction = self

        (prefix, local) = self._name.evaluate(context)
        if prefix is not None:
            name = prefix + ':' + local
        else:
            name = local

        # From sec. 7.1.3 of the XSLT spec,
        #  1. if 'namespace' attr is not present, use ns in scope, based on prefix
        #    from the element QName in the 'name' attr value; if no prefix, use
        #    empty namespace
        #  2. if 'namespace' attr is present and empty string, use empty ns ALWAYS
        #  3. if 'namespace' attr is present, namespace is attr value
        #
        if not self._namespace:
            if prefix is not None:
                if not self.namespaces.has_key(prefix):
                    raise XsltRuntimeException(Error.UNDEFINED_PREFIX,
                                               self, prefix)
                namespace = self.namespaces[prefix]
            else:
                namespace = EMPTY_NAMESPACE

        else:
            namespace_attr_value = self._namespace and self._namespace.evaluate(context)
            if namespace_attr_value == '':
                namespace = EMPTY_NAMESPACE
            else:
                namespace = namespace_attr_value

        processor.pushResult()
        try:
            # FIXME: Add error checking of child nodes
            for child in self.children:
                context = child.instantiate(context, processor)[0]
        finally:
            rtf = processor.popResult()

        processor.writers[-1].attribute(name, rtf.stringValue, namespace)
        return (context,)

