########################################################################
#
# File Name:   TemplateElement.py
#
# Docs:        http://docs.4suite.org/4XSLT/TemplateElement.py.html
#
"""
Implementation of the XSLT Spec template stylesheet element.
WWW: http://4suite.org/4XSLT        e-mail: support@4suite.org

Copyright (c) 1999-2001 Fourthought Inc, USA.   All Rights Reserved.
See  http://4suite.org/COPYRIGHT  for license and copyright information
"""
from pprint import pprint
from xml.dom import Node
from Ft.Xml import EMPTY_NAMESPACE
from Ft.Xml.Xslt import XsltElement, XsltException, Error, XSL_NAMESPACE
from Ft.Xml.Xslt import CategoryTypes, ContentInfo, AttributeInfo

class TemplateElement(XsltElement):
    category = CategoryTypes.TOP_LEVEL_ELEMENT
    content = ContentInfo.Seq(
        ContentInfo.Rep(ContentInfo.QName(XSL_NAMESPACE, 'xsl:param')),
        ContentInfo.Template)
    legalAttrs = {
        'match' : AttributeInfo.Pattern(),
        'name' : AttributeInfo.QName(),
        'priority' : AttributeInfo.Number(),
        'mode' : AttributeInfo.QName(),
        }

    def getTemplateInfo(self):
        infos = []
        if self._match:
            # A 'shortcut' is is tuple of tuples in the form:
            #  ((pattern, axis_type), (node_type, expanded_name))
            shortcuts = self._match.getShortcuts(self.namespaces)
            for ((pattern, axis_type), quick_key) in shortcuts:
                if self._priority is None:
                    priority = pattern.priority
                else:
                    priority = self._priority
                pattern_info = (pattern, axis_type, self)
                shortcut = (self.importIndex, priority, self._mode,
                            pattern_info, quick_key)
                infos.append(shortcut)
        return (infos, self._name)

    def instantiate(self, context, processor, params=None):
        params = params or {}
        while 1:
            context.recursiveParams = None

            for child in self.children:
                if child.expandedName == (XSL_NAMESPACE, 'param'):
                    value = params.get(child._name)
                    if value is not None:
                        context.varBindings[child._name] = value
                    else:
                        context = child.instantiate(context, processor)[0]
                else:
                    context = child.instantiate(context, processor)[0]

            if context.recursiveParams is not None:
                # Update the params from the values given in recursiveParams.
                params.update(context.recursiveParams)
            else:
                break
        return (context,)
