########################################################################
#
# File Name:            __init__.py
#
# Documentation:        http://docs.4suite.org/Lib/__init__.py.html
#
"""
WWW: http://4suite.org/Lib         e-mail: support@4suite.org

Copyright (c) 2000-2001 Fourthought Inc, USA.   All Rights Reserved.
See  http://4suite.org/COPYRIGHT  for license and copyright information
"""

EMPTY_NAMESPACE = None
EMPTY_PREFIX = None
XML_NAMESPACE = "http://www.w3.org/XML/1998/namespace"
XMLNS_NAMESPACE = "http://www.w3.org/2000/xmlns/"
XHTML_NAMESPACE = "http://www.w3.org/1999/xhtml"


from __packageInfo__ import version
__version__ = version
del version


from Ft import FtException

class ReaderException(FtException):
    XML_PARSE_ERROR = 100
    XML_SAX_PARSE_ERROR = 101
    XMLNS_UNKNOWN_PREFIX = 200
    INVALID_XMLNS = 201

    def __init__(self, errorCode, *args):
        FtException.__init__(self, errorCode, MessageSource.READER, args)
        return

class PdomletteException(FtException):
    PICKLE_DOCUMENT = 500
    PICKLE_DOCUMENT_FRAGMENT = 501

    def __init__(self, errorCode, *args):
        FtException.__init__(self, errorCode, MessageSource.PDOMLETTE, args)
        return

class XIncludeException(FtException):
    XINCLUDE_MISSING_HREF = 10
    CIRCULAR_INCLUDE_ERROR = 20

    def __init__(self, errorCode, *args):
        FtException.__init__(self, errorCode, MessageSource.XINCLUDE, args)
        return

def PySplitQName(qname):
    """
    Input a QName according to XML Namespaces 1.0
    http://www.w3.org/TR/REC-xml-names
    Return the name parts according to the spec
    """
    index = qname.rfind(':')
    if index == -1:
        split_name = (None, qname)
    else:
        prefix = qname[:index]
        local = qname[index+1:]
        split_name = (prefix, local)
    return split_name

try:
    import SplitQNamec
    SplitQName = SplitQNamec.SplitQName
except ImportError:
    #Do it this way so we can test both!!!
    SplitQName = PySplitQName

import MessageSource

from distutils import version
pyxml_required = version.StrictVersion('0.8.0')

def CheckVersion(feature = None):
    try:
        import _xmlplus
        xml_version = version.StrictVersion(_xmlplus.__version__)
    except:
        xml_version = version.StrictVersion('0.0.0')

    if xml_version < pyxml_required:
        import sys
        if feature:
            feature_string = "%s in " % feature
        else:
            feature_string = "this feature in "
        print """
        PyXML v%s is required for %s4Suite.
        It is available at http://sourceforge.net/projects/pyxml.
        """ % (str(pyxml_required),feature_string)
        sys.exit(1)



#Wrap this so that we can import it later
def ApplyXupdate(*args, **kw_args):
    import Ft.Xml.XUpdate
    return apply(XUpdate.ApplyXUpdate,agrs,kw_args)



def VerifyPyexpat(name, dtdSupport=0):
    """
    Some of our modules still need pyexpat installed.  This is a common
    function to test that it is properly installed and can be used.
    """
    try:
        from xml.parsers import expat
        expat.ParserCreate
    except (ImportError, AttributeError):
        raise SystemExit("You must either install PyXML v%s or recompile "
                         "your python with PyExpat support to use '%s'" %
                         (pyxml_required, name))

    if dtdSupport:
        # DTD support requires expat 1.95.0 or later
        if hasattr(expat, 'model'):
            # We have a pyexpat that we can use!
            return
        raise SystemExit("The expat library used for pyexpat must be at least "
                         "version 1.95.0.  You must either install PyXML v%s "
                         "or compile python with an updated expat library." %
                         pyxml_required)

##4XML string processing design considerations


##1) 4XSLT and 4XPath (indeed 4XPointer, 4XLink and 4RDF as well) will use
##unicode objects for all internal string processing

##* Note that this principle isn't yet all in place.  For instance,
##Jeremy's parsers, 4RDF and some other parts will need extensive work to
##conform to this.  Most of my effort has been in normalizing the
##boundaries between code that is compliant with and non-compliant with
##this principle.

##2) The public programmer's API will deal only with Unicode objects and
##ASCII-encoded strings

##* I actually would like to accept only Unicode objects to avoid any
##English bias, but Jeremy argues for accepting ASCII strings

##3) Input from file, URI, etc. will accept properly encoded character
##buffers

##* Note: this is implemented by the codec's streamreader

##4) Output APIs to Python strings will output Unicode objects

##5) Output APIs to file objects will put out character buffers with the
##appropriate encoding based on the processing (e.g. xsl:output element)

##* Note: this is implemented by the codec's streamwriter

##6) In XML and HTML output, named character entities will be preferred
##for output, followed by the natural repr from the encoding, with a
##fall-back to numerical character entities

