########################################################################
#
# File Name:            cDomlette.py
#
# Documentation:        http://docs.4suite.org/4Suite/cDomlette.py.html
#
"""
WWW: http://4suite.org/         e-mail: support@4suite.org

Copyright (c) 2000-2001 Fourthought Inc, USA.   All Rights Reserved.
See  http://4suite.org/COPYRIGHT  for license and copyright information
"""
import os, re
import cDomlettec
import Ft.Lib
from Ft.Lib import Uri
#from Ft.Xml import ReaderBase


##ElementType = cDomlettec.ElementType
##DocumentType = cDomlettec.DocumentType
##CommentType = cDomlettec.CommentType
##ProcessingInstructionType = cDomlettec.ProcessingInstructionType
##TextType = cDomlettec.TextType
##CommentType = cDomlettec.CommentType

##cDomletteTypes = [ElementType,
##                  DocumentType,
##                  CommentType,
##                  ProcessingInstructionType,
##                  TextType,
##                  CommentType,
##                  ]


#For the interface
class Node:
    ELEMENT_NODE                = 1
    ATTRIBUTE_NODE              = 2
    TEXT_NODE                   = 3
    CDATA_SECTION_NODE          = 4
    ENTITY_REFERENCE_NODE       = 5
    ENTITY_NODE                 = 6
    PROCESSING_INSTRUCTION_NODE = 7
    COMMENT_NODE                = 8
    DOCUMENT_NODE               = 9
    DOCUMENT_TYPE_NODE          = 10
    DOCUMENT_FRAGMENT_NODE      = 11
    NOTATION_NODE               = 12


implementation = cDomlettec.implementation


def nonvalParse(isrc):
    try:
        return cDomlettec.parse(isrc, 1)
    except SyntaxError, e:
        #print e.__dict__
        fields = str(e).split('\t')
        if len(fields) < 4:
            raise
        from Ft.Xml import ReaderException
        raise ReaderException(ReaderException.XML_PARSE_ERROR,
                              fields[0],
                              int(fields[1]),
                              int(fields[2]),
                              '\t'.join(fields[3:]))
    except cDomlettec.error, e:
        #print e.__dict__
        fields = str(e).split('\t')
        if len(fields) < 4:
            raise
        from Ft.Xml import ReaderException
        raise ReaderException(ReaderException.XML_PARSE_ERROR,
                              fields[0],
                              int(fields[1]),
                              int(fields[2]),
                              '\t'.join(fields[3:]))

#NOTE - because cDomlette does not yet have a validating parser, we
# need to use minidom
from FtMiniDom import valParse


XmlStrStrip = cDomlettec.XmlStrStrip
XmlStrRStrip = cDomlettec.XmlStrRStrip
XmlStrLStrip = cDomlettec.XmlStrLStrip


# Functions used for testing
TestTree = cDomlettec.TestTree
TestRefCounts = cDomlettec.TestRefCounts
StartNodeCounting = cDomlettec.StartNodeCounting
GetNodeCount = cDomlettec.GetNodeCount


# Fragment processing

class FragmentSpec:
    def __init__(self):
        self.states = []
        self.nextAvailStateId = g_nextAvailStateId
        self.fromState = g_initialState
        return

#FIXME: these constants are copied from Ft/Xml/src/domlette/state_machine.h
#They should be instead set by code in the C expat handler to ensure sync
g_nextAvailStateId = 100
g_initialState = 2

ELEMENT_MATCH = 1
ELEMENT_COUNT = 2
ATTRIBUTE_MATCH = 3


def ProcessFragment(frag):
    """
    Take an XPointer fragment and return a structure suitable for the
    cDomlette parser to update state tables
    Xptr e.g. xmlns(x=http://uche.ogbuji.net/eg) xpointer(/x:spam/x:eggs)
    """
    from Ft.Xml.XPath import Evaluate, Compile, Context
    from Ft.Xml.XPath.ParsedAbsoluteLocationPath import ParsedAbsoluteLocationPath
    from Ft.Xml.XPath.ParsedRelativeLocationPath import ParsedRelativeLocationPath
    from Ft.Xml import EMPTY_NAMESPACE, XML_NAMESPACE
    XPTR_EXPR = re.compile("#.*xpointer\((?P<xptr>.*?)\)")
    NS_EXPR = re.compile("xmlns\((?P<ns>.*?)\)")
    xptr = XPTR_EXPR.findall(frag)
    #There must be an xptr expression
    if not xptr: return None

    xptr = xptr[0]
    ns_mappings = {u'xml': XML_NAMESPACE}
    ns_matches = NS_EXPR.findall(frag)
    for m in ns_matches:
        #FIXME: Check for bogus xml ns decl
        prefix, ns = m.split('=')
        ns_mappings[prefix] = ns
    expr = Compile(xptr)
    #Assume context = root, which means in effect that the absolute = its relative "child"
    if isinstance(expr, ParsedAbsoluteLocationPath):
        #Extract the relative part (which looks like one form of parsed step)
        expr = expr._child
    result = FragmentSpec()
    HandleStep(expr, result, ns_mappings)
    return result


def HandleStep(expr, spec, nss):
    from Ft.Xml.XPath import Util
    from Ft.Xml import EMPTY_NAMESPACE, XML_NAMESPACE
    from Ft.Xml.XPath.ParsedExpr import ParsedNLiteralExpr, ParsedEqualityExpr
    if hasattr(expr, "_left"):
        lstep = expr._left
        curr_step = expr._right
        HandleStep(lstep, spec, nss)
    elif hasattr(expr, "_left"):
        #Down to the "last" recursive step
        curr_step = expr._child
    else:
        #Down to the "last" recursive step
        curr_step = expr
    #Set criteria by expanded name
    element_exp_name = Util.ExpandQName(curr_step._nodeTest._name, namespaces=nss)
    criteria = [(ELEMENT_MATCH,) + element_exp_name]
    #Set criteria from predicates
    if curr_step._predicates:
        pred = curr_step._predicates._predicates[0]
        if isinstance(pred, ParsedNLiteralExpr):
            #The third item is a counter used to count elements during the parsing
            criteria.extend([(ELEMENT_COUNT, int(pred._literal), [1])])
        elif isinstance(pred, ParsedEqualityExpr) and pred._op == u'=':
            if hasattr(pred._left, '_axis') and \
                   pred._left._axis._axis == 'attribute':
                # criteria code
                criterion = [ATTRIBUTE_MATCH]

                # Add the expanded name
                if hasattr(pred._left._nodeTest, '_localName'):
                    criterion.append(nss[pred._left._nodeTest._prefix])
                    criterion.append(pred._left._nodeTest._localName)
                else:
                    criterion.append(None)
                    criterion.append(pred._left._nodeTest._name)

                # Add the expected value
                criterion.append(pred._right._literal)

                # Add this information to the criteria
                criteria.append(tuple(criterion))

    # Add state transitions for the current step
    spec.states.append((spec.fromState, spec.nextAvailStateId,
                        spec.nextAvailStateId + 1, spec.nextAvailStateId + 2,
                        criteria
                        ))
    spec.fromState = spec.nextAvailStateId
    spec.nextAvailStateId += 3
    return


