########################################################################
#
# File:                       __init__.py
#
"""
WWW: http://4suite.org        e-mail: support@4suite.org

Copyright (c) 2001 Fourthought Inc, USA.   All Rights Reserved.
See  http://4suite.org/COPYRIGHT  for license and copyright information
"""

__revision__ = "$Id: __init__.py,v 1.19 2002/09/30 18:00:57 jkloth Exp $"

import types
STRING_TYPES = [types.StringType, types.UnicodeType]
del types

# Determine charset
DEFAULT_ENCODING = 'US-ASCII'
try:
    import locale
except ImportError:
    pass 
else:
    DEFAULT_ENCODING = locale.getlocale()[1]
    if DEFAULT_ENCODING is None:
        #Get around a Python bug on some platforms
        #see http://mail.python.org/pipermail/python-bugs-list/2002-July/012505.html
        try:
            DEFAULT_ENCODING = locale.getdefaultlocale()[1]
        except:
            # Unable to determine platform encoding, only allow US-ASCII
            pass 
    del locale

class FtException(Exception):
    def __init__(self, errorCode, messages, args):
        # By defining __str__, args will be available.  Otherwise
        # the __init__ of Exception sets it to the passed in arguments.
        self.params = args
        self.errorCode = errorCode
        self.message = messages[errorCode] % args
        Exception.__init__(self, self.message, args)

    def __str__(self):
        return self.message


import os
FT_DIR = os.path.dirname(os.path.abspath(__file__))
SHARE_DIR = os.path.join(FT_DIR, 'Share')
PACKAGE_DATABASE = os.path.join(FT_DIR, 'Share', 'Data', 'packages.db')

if os.environ.get('FT_IN_SETUP') != 'yes':
    try:
        # Any C extension from Ft.Lib will do here as Ft is installed
        # with the Lib package.
        from Ft.Lib import boolean
    except:
        msg = """
4Suite is having trouble importing the modules it needs. \
This is usually because the current working directory, which happens \
to be %s at the moment, contains modules with names that are the \
same as modules that 4Suite is trying to import. For example, 4Suite \
cannot be invoked from the source code directory that contains the \
setup.py that was used to install 4Suite.

Try changing the current working directory to a suitable location \
outside of the 4Suite source. If you continue to have trouble, \
please send a message to the 4Suite mailing list at \
4suite@lists.fourthought.com, along with any information that might \
explain why you got this message.
""" % os.getcwd()

        # word wrap with line-break preservation
        def wrap(s, width=78):
            return reduce(lambda line, word,width=width: "%s%s%s" % 
                          (line,
                           ' \n'[(len(line[line.rfind('\n')+1:])
                                 + len(word) >= width)],
                           word),
                          s.split(' ')
                         )
        raise SystemExit(wrap(msg,78))
    else:
        del boolean


del os

